
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
#pragma warn -hid

#include "Modem.h"
#include "modemselect.h"
#pragma link "ModemSelect"
#pragma link "ModemProgress"
TModem *ptrModem;
//---------------------------------------------------------------------------
static inline TModem *ValidCtrCheck()
{
	return new TModem(NULL);
}
/****************************************************************************

function TModem

TModem component constructor

entry
exit

****************************************************************************/
__fastcall TModem::TModem(TComponent* Owner)
	: TComponent(Owner)
{
	ptrModem = this;
	Connected = false;
	DeviceCount = 0;
	DeviceID = 0;
	ModemList = new TStringList();
	Call = NULL;
	DialogOpen = false;
	ProgressDlg = new TModemProgressDlg(this);
	ProgressDlgOpen = false;
// set default values for properties
	FStatus = msIdle;
	FUart = NULL;
// init TAPI
	strnset((LPSTR)&CallParams, '0', sizeof(CallParams));
	CallParams.dwTotalSize = sizeof(CallParams);
	CallParams.dwBearerMode = LINEBEARERMODE_VOICE;
	CallParams.dwMediaMode = LINEMEDIAMODE_DATAMODEM;
	if(lineInitialize(&LineApp, HInstance, LineCallback,
						(LPCSTR)Application->ExeName.c_str(), &DeviceCount) < 0)
		LineApp = 0;
	else if(DeviceCount == 0)
	{
		lineShutdown(LineApp);
		LineApp = 0;
	}
	if(LineApp != 0) BuildDeviceList();
}
/****************************************************************************

function ~TModem

TModem component destructor

entry
exit

****************************************************************************/
__fastcall TModem::~TModem()
{
	if(Line) lineClose(Line);
	if(LineApp) lineShutdown(LineApp);
	if(Connected) Shutdown();
	delete ModemList;
	delete ProgressDlg;
}

//---------------------------------------------------------------------------
namespace Modem
{
	void __fastcall Register()
	{
		TComponentClass classes[1] = {__classid(TModem)};
		RegisterComponents("MyStuff", classes, 0);
	}
}

/****************************************************************************

function LineCallback

Callback function for TAPI

entry
exit

****************************************************************************/
HCALL CallToAnswer;
void CALLBACK LineCallback(	DWORD hDevice, DWORD dwMsg, DWORD dwCallbackInstance,
							DWORD dwParam1, DWORD dwParam2, DWORD dwParam3)
{
HCALL hCall;
char LineCallBuf[1024];
LINECALLSTATUS CallStatus;					// holds status of call
DWORD Rings;

	switch(dwMsg)
	{
		case LINE_REPLY:
//			if(dwParam1 == AnswerID)
//				ptrModem->SetStatus(dwParam2, " Answer Reply");
//			break;
// status of line has changed
		case LINE_LINEDEVSTATE:
			switch(dwParam1)
			{
				case LINEDEVSTATE_RINGING:
					if(dwParam3 >= ptrModem->RingsToAnswer)
					{
						lineAnswer(CallToAnswer, NULL, 0);
						ptrModem->SetStatus(msAnswering, "Answering");
					}
					break;
			}
			break;
// status of call has changed
		case LINE_CALLSTATE:
			hCall = (HCALL)hDevice;
			switch(dwParam1)
			{
			// call has gone idle
				case LINECALLSTATE_IDLE:
					CallToAnswer = NULL;
					ptrModem->SetStatus(msIdle, "Idle");
					if(hCall)
					{
					// free the memory associated with the call
						lineDeallocateCall(hCall);
						ptrModem->Shutdown();
					}
					break;
			// call is incoming
				case LINECALLSTATE_OFFERING:
					CallToAnswer = hCall;
					ptrModem->SetStatus(msRinging, "Ringing");
					break;
			// call has connected
				case LINECALLSTATE_CONNECTED:
					if(hCall)
					{
						ptrModem->SetStatus(msConnected, "Connected");
						((LPLINECALLINFO)LineCallBuf)->dwTotalSize = sizeof(LineCallBuf);
						if(lineGetCallInfo(hCall, (LPLINECALLINFO)LineCallBuf) == 0)
							ptrModem->Startup();
					}
					break;
			// dialling is complete and call is on it's way
				case LINECALLSTATE_PROCEEDING:
					ptrModem->SetStatus(msWaitReply, "Waiting for reply");
					break;
			// phone is dialling
				case LINECALLSTATE_DIALING:
					ptrModem->SetStatus(msDialling, String("Dialling ") + ptrModem->TelNo);
					break;
			// line was busy
				case LINECALLSTATE_BUSY:
					ptrModem->SetStatus(msBusy, "Busy");
					break;
			// call has disconnected
				case LINECALLSTATE_DISCONNECTED:
					ptrModem->SetStatus(msDisconnected, "Disconnected");
				// disconnect call if required
					if(hCall) lineDrop(hCall, NULL,0);
				// shutdown terminal if connected
					ptrModem->Shutdown();
					break;
				default:
//					ptrModem->SetStatus(dwParam1, " Param1");
					break;
			}
			break;
	}
}

/****************************************************************************

function setstatus

Updates our line status and calls the user event handler

entry	TDCModemStatus stat
exit	void

****************************************************************************/
void __fastcall TModem::SetStatus(TDCModemStatus stat, String s)
{
	if(stat != FStatus)
	{
		FStatus = stat;
		if(!OnStatusChange) return;
		OnStatusChange(this, stat, s);
		if(ProgressDlg->Visible)
			ProgressDlg->SetStatus(stat, s);
	}
}

/****************************************************************************

function shutdown

Shuts down the modem connection and frees the threads

entry	void
exit	void

****************************************************************************/
void __fastcall TModem::Shutdown()
{
//	ProgressDlg->Hide();
	if(Connected == false) return;
	if(FUart) FUart->ReleaseHandle();
	CloseHandle(hCommDevice);
	Connected = false;
	if(OnDisconnect) OnDisconnect(this, FStatus);
}

/****************************************************************************

function Startup

Starts up the port connection after the modem has connected to the remote
modem.

entry	void
exit	void

****************************************************************************/
void __fastcall TModem::Startup()
{
MODEM_INFO ModemInfo;

//	ProgressDlg->Hide();
	if(Connected) return;
	ModemInfo.vs.dwTotalSize = sizeof(MODEM_INFO);
	ModemInfo.vs.dwStringFormat = STRINGFORMAT_BINARY;
// get the device information of the line which is connected to a modem and
// put the info into ModemInfo
// result < 0 = error
	if(lineGetID(Line, 0, 0, LINECALLSELECT_LINE, (LPVARSTRING)&ModemInfo,
				"comm/datamodem") < 0)
	{
	// report error and exit
// TODO... setup error code
		return;
	}
// the handle to the comms device is contained in ModemInfo
	hCommDevice = ModemInfo.hComm;
// check for invalid comms handle
	if(hCommDevice == INVALID_HANDLE_VALUE)
	{
// TODO... setup error code
		return;
	}
// terminal is now connected
	if(FUart) FUart->AssignHandle(hCommDevice);
	Connected = true;
	if(OnConnect) OnConnect(this, FStatus);
}

/****************************************************************************

function builddevicelist

Builds a list of all the modem devices attached to the PC

entry	void
exit	void

****************************************************************************/
void __fastcall TModem::BuildDeviceList()
{
DWORD x;
MODEM_INFO ModemInfo;
LINEEXTENSIONID extid;
int index;

	ModemList->Clear();
	ModemInfo.vs.dwTotalSize = sizeof(MODEM_INFO);
	ModemInfo.vs.dwStringFormat = STRINGFORMAT_BINARY;
	for(x = 0; x < DeviceCount; x++)
	{
		if(lineNegotiateAPIVersion(LineApp, x, 0x00010000, 0x00090000,
									&tapiVersion, &extid) < 0)
			return;
		else if(lineOpen(LineApp, x, &Line, tapiVersion, 0, 0,
							LINECALLPRIVILEGE_NONE, 0, &CallParams) < 0)
			continue;
// set size of modem info buffer
		ModemInfo.vs.dwTotalSize = sizeof(MODEM_INFO);
		ModemInfo.vs.dwStringFormat = STRINGFORMAT_BINARY;
		memset(ModemInfo.szDeviceName, 0, 255);
// get device ID
		if(lineGetID(Line, 0, 0, LINECALLSELECT_LINE, (LPVARSTRING)&ModemInfo,
					"comm/datamodem") < 0)
			continue;
		if(ModemInfo.szDeviceName[0])
		{
			index = ModemList->Add(ModemInfo.szDeviceName);
			ModemList->Objects[index] = (TObject *)x;
		}
		lineClose(Line);
	}
	if(ModemList->Count)
	{
		DeviceIndex = 0;
		DeviceID = (DWORD)ModemList->Objects[0];
	}
}

/****************************************************************************

function dial

Dials and connects a remote modem

entry	String telno
exit	bool true or false

****************************************************************************/
bool __fastcall TModem::Dial(String No, bool UseDlg/* = false*/)
{
LINEEXTENSIONID extid;

	if(No.Length() == 0) return false;
	if(Line) lineClose(Line);
	TelNo = No;
	if(UseDlg == true)
	{
		return (ProgressDlg->ShowModal() == mrOk);
	}
	SetStatus(msConnecting,String("Connecting to device: ") + ModemList->Strings[Device]);
// App and TAPI must negotiate to agree on a TAPI version (< 0 = error)
	if(lineNegotiateAPIVersion(LineApp, DeviceID, 0x00010000, 0x00090000,
								&tapiVersion, &extid) < 0)
		return false;
// open the line using a device that supports the parameters in the CallParams (<0 = error)
	if(lineOpen(LineApp, DeviceID, &Line, tapiVersion, 0, 0,
				LINECALLPRIVILEGE_NONE, 0, &CallParams) < 0)
		return false;
// make the call (<0 = error)
	if(lineMakeCall(Line, &Call, No.c_str(), 0, &CallParams) < 0)
		return false;
// open dialog
//	if(UseDlg == true)
//	{
//		ProgressDlg->Show();
//		ProgressDlg->SetStatus(String("Dialling ") + No);
//	}
	return true;
}

/****************************************************************************

function hangup

Hangs up the current call

entry	void
exit	void

****************************************************************************/
void __fastcall TModem::Hangup()
{
	if(Call == NULL) return;
	lineDrop(Call, NULL, 0);
	lineClose(Line);
	Shutdown();
}

/****************************************************************************

function listen

Enabled the TAPI device to listen for incoming calls

entry	DWORD number of rings after which to answer
exit	bool true or false

****************************************************************************/
bool __fastcall TModem::Listen(DWORD Rings/* = 3*/)
{
LINEEXTENSIONID extid;

	if(Line) lineClose(Line);
// App and TAPI must negotiate to agree on a TAPI version (< 0 = error)
	if(lineNegotiateAPIVersion(LineApp, DeviceID, 0x00010000, 0x00090000,
								&tapiVersion, &extid) < 0)
		return false;
// open the line using a device that supports the parameters in the CallParams (<0 = error)
	if(lineOpen(LineApp, DeviceID, &Line, tapiVersion, 0, 0,
				LINECALLPRIVILEGE_OWNER,
				LINEMEDIAMODE_DATAMODEM, &CallParams) < 0)
		return false;
// tell TAPI we want RING messages
	if(lineSetStatusMessages(Line, LINEDEVSTATE_RINGING, 0) < 0)
		return false;
// setup ring count
	RingsToAnswer = Rings;
	return true;
}

/****************************************************************************

function setdevice

Sets the device type for calls

entry	DWORD device type (index of device in list)
exit	void

****************************************************************************/
void __fastcall TModem::SetDevice(DWORD dev)
{
	if(dev >= DeviceCount) return;
	DeviceIndex = dev;
	DeviceID = (DWORD)ModemList->Objects[DeviceIndex];
	if(DialogOpen == true && OnDeviceChange != NULL)
		OnDeviceChange(this, DeviceIndex);
}

/****************************************************************************

function retuerns the handle to the comms port

entry
exit	HANDLE

****************************************************************************/
HANDLE __fastcall TModem::GetHandle(void)
{
	if(Connected == false) return INVALID_HANDLE_VALUE;
	else return hCommDevice;
}

/****************************************************************************

function UserConfig

Allows the user to configure the currently selected device

entry	void
exit	void

****************************************************************************/
bool __fastcall TModem::UserConfigure(HWND h /* = NULL*/)
{
LINEEXTENSIONID extid;

	if(Connected == true) return false;
// App and TAPI must negotiate to agree on a TAPI version (< 0 = error)
	if(lineNegotiateAPIVersion(LineApp, DeviceID, 0x00010000, 0x00090000,
								&tapiVersion, &extid) < 0)
		return false;
// open the line using a device that supports the parameters in the CallParams (<0 = error)
	else if(lineOpen(LineApp, DeviceID, &Line, tapiVersion, 0, 0,
						LINECALLPRIVILEGE_NONE, 0, &CallParams) < 0)
		return false;
// open dialog
	if(h == NULL) h = Application->Handle;
	lineConfigDialog(DeviceID, h, NULL);
// close line again
	lineClose(Line);
	return true;
}

/****************************************************************************

function

entry
exit

****************************************************************************/
void __fastcall TModem::UserSelectModem()
{
TModemSelectDlg *dlg;

	dlg = new TModemSelectDlg(this);
	dlg->pModem = this;
	dlg->DeviceIndex = DeviceIndex;
	DialogOpen = true;
	dlg->ShowModal();
	DialogOpen = false;
	delete dlg;
}

/****************************************************************************

function

entry
exit

****************************************************************************/

/****************************************************************************

function

entry
exit

****************************************************************************/

/****************************************************************************

function

entry
exit

****************************************************************************/

