//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#include <inifiles.hpp>
#pragma hdrstop
#pragma warn -hid

#include "Modem.h"
#pragma link "ModemSelect"
#pragma link "ModemProgress"
#pragma link "DialOptions"

TModem *ptrModem;
//---------------------------------------------------------------------------
static inline TModem *ValidCtrCheck()
{
	return new TModem(NULL);
}
/****************************************************************************

function TModem

TModem component constructor

entry
exit

****************************************************************************/
__fastcall TModem::TModem(TComponent* Owner)
	: TComponent(Owner)
{
	hCommDevice = INVALID_HANDLE_VALUE;
	ptrModem = this;
	Connected = false;
	DeviceCount = 0;
	DeviceID = 0;
	ModemList = new TStringList();
	Call = NULL;
	DialogOpen = false;
	ProgressDlg = new TModemProgressDlg(this);
	ProgressDlgOpen = false;
	Listening = false;
// set default values for properties
	FStatus = msIdle;
	FUart = NULL;
	FManualDial =
	FTerminalBefore =
	FTerminalAfter = moDefault;
	FUseDialDlg =
	FShowDetails = true;
	FUseDialProp = true;
	FAreaCode = "";
	FOutsideLine = "";
// init TAPI
	strnset((LPSTR)&CallParams, '\0', sizeof(CallParams));
	CallParams.dwTotalSize = sizeof(CallParams);
	CallParams.dwBearerMode = LINEBEARERMODE_VOICE;
	CallParams.dwMediaMode = LINEMEDIAMODE_DATAMODEM;
// to overcome a bug in Win95's TAPI DLL, we need to close a lineapp if it was open
// from a previous session
TIniFile *t;
	t = new TIniFile("TMODEM.HLA");
	if(t)
	{
		LineApp = (HLINEAPP)t->ReadInteger("Line Applications","Current Line", 0);
		if(LineApp) lineShutdown(LineApp);
		delete t;
	}
// end of bugfix
	if(lineInitialize(&LineApp, HInstance, LineCallback,
						(LPCSTR)Application->ExeName.c_str(), &DeviceCount) < 0)
		LineApp = 0;
	else if(DeviceCount == 0)
	{
		lineShutdown(LineApp);
		LineApp = 0;
	}
// save the line app to an INI file in case of program reset
	t = new TIniFile("TMODEM.HLA");
	if(t)
	{
		t->WriteInteger("Line Applications","Current Line", (int)LineApp);
		delete t;
	}
//
	if(LineApp != 0)
	{
		BuildDeviceList();
		GetCurrentCountryID();		/* fill in current country			*/
		GetCurrentLocationID();		/* fill in location and area code	*/
	}
}
/****************************************************************************

function ~TModem

TModem component destructor

entry
exit

****************************************************************************/
__fastcall TModem::~TModem()
{
TIniFile *t;

	if(Line) lineClose(Line);
	if(LineApp)
	{
		lineShutdown(LineApp);
// we shutdown successfully, so remove lineapp handle from ini file
		t = new TIniFile("TMODEM.HLA");
		if(t)
		{
			t->EraseSection("Line Applications");
			delete t;
		}
	}
	if(Connected) Shutdown();
	delete ModemList;
	delete ProgressDlg;
}

//---------------------------------------------------------------------------
namespace Modem
{
	void __fastcall Register()
	{
		TComponentClass classes[1] = {__classid(TModem)};
		RegisterComponents("MyStuff", classes, 0);
	}
}

/****************************************************************************

function LineCallback

Callback function for TAPI

entry
exit

****************************************************************************/
HCALL CallToAnswer;
void CALLBACK LineCallback(	DWORD hDevice, DWORD dwMsg, DWORD dwCallbackInstance,
							DWORD dwParam1, DWORD dwParam2, DWORD dwParam3)
{
HCALL hCall;
char LineCallBuf[1024];

	switch(dwMsg)
	{
		case LINE_CLOSE:
				ptrModem->SetStatus(msLineClose, "Line Closed");
				ptrModem->Line = 0;
				break;
		case LINE_REPLY:
			if((long)dwParam2 < 0)
			{
				ptrModem->SetStatus(msError, "Modem Line Error");
			}
			else ptrModem->SetStatus(msOk, "Success");
			break;
// status of line has changed
		case LINE_LINEDEVSTATE:
			switch(dwParam1)
			{
				case LINEDEVSTATE_RINGING:
					if(dwParam3 >= ptrModem->RingsToAnswer)
					{
						lineAnswer(CallToAnswer, NULL, 0);
						ptrModem->SetStatus(msAnswering, "Answering");
					}
					break;
			}
			break;
// status of call has changed
		case LINE_CALLSTATE:
			hCall = (HCALL)hDevice;
			switch(dwParam1)
			{
			// call has gone idle
				case LINECALLSTATE_IDLE:
					CallToAnswer = NULL;
					ptrModem->SetStatus(msIdle, "Idle");
					if(hCall)
					{
					// free the memory associated with the call
//						ptrModem->Shutdown();
						lineDeallocateCall(hCall);
						ptrModem->FreeHandle();
					}
					break;
			// call is incoming
				case LINECALLSTATE_OFFERING:
					CallToAnswer = hCall;
					ptrModem->SetCallProperties();
					ptrModem->SetStatus(msRinging, "Ringing");
					break;
			// call has connected
				case LINECALLSTATE_CONNECTED:
					if(hCall)
					{
						ptrModem->SetStatus(msConnected, "Connected");
						((LPLINECALLINFO)LineCallBuf)->dwTotalSize = sizeof(LineCallBuf);
						if(lineGetCallInfo(hCall, (LPLINECALLINFO)LineCallBuf) == 0)
						{
							ptrModem->Startup();
//							Call = hCall;
						}
//						else Call = NULL;
					}
					break;
			// dialling is complete and call is on it's way
				case LINECALLSTATE_PROCEEDING:
					ptrModem->SetStatus(msWaitReply, "Waiting for reply");
					break;
			// phone is dialling
				case LINECALLSTATE_DIALING:
					ptrModem->SetStatus(msDialling, String("Dialling ") + ptrModem->FriendlyNo);
					break;
			// line was busy
				case LINECALLSTATE_BUSY:
					ptrModem->SetStatus(msBusy, "Busy");
					break;
			// call has disconnected
				case LINECALLSTATE_DISCONNECTED:
				// disconnect call if required
					if(hCall) lineDrop(hCall, NULL,0);
				// shutdown terminal if connected
					ptrModem->SetStatus(msDisconnected, "Disconnected");
					ptrModem->Shutdown();
					break;
				default:
					break;
			}
			break;
	}
}

/****************************************************************************

function setstatus

Updates our line status and calls the user event handler

entry	TDCModemStatus stat
exit	void

****************************************************************************/
void __fastcall TModem::SetStatus(TDCModemStatus stat, String s)
{
	if(stat != FStatus)
	{
		FStatus = stat;
		if(OnStatusChange) OnStatusChange(this, stat, s);
		if(ProgressDlg->Visible) ProgressDlg->SetStatus(stat, s);
	}
}

/****************************************************************************

function shutdown

Shuts down the modem connection

entry	void
exit	void

****************************************************************************/
void __fastcall TModem::Shutdown()
{
	if(Connected == false) return;
	if(FUart) FUart->ReleaseHandle();
	Connected = false;
	if(OnDisconnect) OnDisconnect(this, FStatus);
}

/****************************************************************************

function Startup

Starts up the port connection after the modem has connected to the remote
modem.

entry	void
exit	void

****************************************************************************/
void __fastcall TModem::Startup()
{
MODEM_INFO ModemInfo;

	if(Connected) return;
	ModemInfo.vs.dwTotalSize = sizeof(MODEM_INFO);
	ModemInfo.vs.dwStringFormat = STRINGFORMAT_BINARY;
// get the device information of the line which is connected to a modem and
// put the info into ModemInfo
// result < 0 = error
	if(lineGetID(Line, 0, 0, LINECALLSELECT_LINE, (LPVARSTRING)&ModemInfo,
				"comm/datamodem") < 0)
	{
		return;
	}
// the handle to the comms device is contained in ModemInfo
	hCommDevice = ModemInfo.hComm;
// check for invalid comms handle
	if(hCommDevice == INVALID_HANDLE_VALUE)
	{
		return;
	}
// terminal is now connected
	if(FUart) FUart->AssignHandle(hCommDevice);
	Connected = true;
	if(OnConnect) OnConnect(this, FStatus);
}

/****************************************************************************

function builddevicelist

Builds a list of all the modem devices attached to the PC

entry	void
exit	void

****************************************************************************/
void __fastcall TModem::BuildDeviceList()
{
DWORD x, needed;
LPLINEDEVCAPS DevCaps;
LINEEXTENSIONID extid;
int index;
long result;

	ModemList->Clear();
	for(x = 0; x < DeviceCount; x++)
	{
		if(lineNegotiateAPIVersion(LineApp, x, 0x00010000, 0x00090000,
									&tapiVersion, &extid) < 0)
			continue;
		DevCaps = (LPLINEDEVCAPS)new BYTE[sizeof(LINEDEVCAPS)];
		memset(DevCaps, 0, sizeof(LINEDEVCAPS));
		DevCaps->dwTotalSize = sizeof(LINEDEVCAPS);
		result = lineGetDevCaps(LineApp, x, tapiVersion, 0, DevCaps);
		needed = DevCaps->dwNeededSize;
		delete DevCaps;
		if(result < 0 && result != (long)LINEERR_STRUCTURETOOSMALL)
			continue;
		DevCaps = (LPLINEDEVCAPS)new BYTE[needed];
		memset(DevCaps, 0, needed);
		DevCaps->dwTotalSize = needed;
		if(lineGetDevCaps(LineApp, x, tapiVersion, 0, DevCaps) < 0)
		{
			delete DevCaps;
			continue;
		}
		if(DevCaps->dwMediaModes & LINEMEDIAMODE_DATAMODEM)
		{
			index = ModemList->Add((char*)DevCaps + DevCaps->dwLineNameOffset);
			ModemList->Objects[index] = (TObject *)x;
		}
		delete DevCaps;
	}
	if(ModemList->Count)
	{
		DeviceIndex = 0;
		DeviceID = (DWORD)ModemList->Objects[0];
	}
}

/****************************************************************************

function dial

Dials and connects a remote modem

entry	void
exit	TDCModemResult

****************************************************************************/
TDCModemResult __fastcall TModem::Dial()
{
	return Dial(PhoneNo);
}

/****************************************************************************

function dial

Dials and connects a remote modem

entry	String telno
exit	TDCModemResult

****************************************************************************/
TDCModemResult __fastcall TModem::Dial(String No)
{
LINEEXTENSIONID extid;
String NewNo;

	if(DeviceCount == 0) return mError;
	if(No.Length() == 0) return mError;
	if(Line) lineClose(Line);
	Listening = false;
	TelNo = No;
	FriendlyNo = No;
	if(FUseDialDlg == true)
	{
		ProgressDlgOpen = true;
		ProgressDlg->ShowDetails = FShowDetails;
		ProgressDlg->ShowModal();
		FShowDetails = ProgressDlg->ShowDetails;
		ProgressDlgOpen = false;
		return DialResult;
	}
	SetStatus(msConnecting,String("Connecting to device: ") + ModemList->Strings[Device]);
// App and TAPI must negotiate to agree on a TAPI version (< 0 = error)
	if(lineNegotiateAPIVersion(LineApp, DeviceID, 0x00010000, 0x00090000,
								&tapiVersion, &extid) < 0)
		return mError;
// setup manual dialling and terminal options
	SetCallProperties();
// open the line using a device that supports the parameters in the CallParams (<0 = error)
	if(lineOpen(LineApp, DeviceID, &Line, tapiVersion, 0, 0,
				LINECALLPRIVILEGE_NONE, 0, &CallParams) < 0)
		return mError;
// translate dest number if system dial properties should be used
	if(FUseDialProp == true)
		TelNo = GetDiallableNumber(Country, AreaCode, TelNo);
	else
	{
		TelNo = "";
		if(FOutsideLine != "") TelNo = TelNo + FOutsideLine + " ";
		FriendlyNo = TelNo;
		if(FAreaCode != "")
		{
			TelNo = TelNo + FAreaCode;
			FriendlyNo = FriendlyNo + "(" + FAreaCode + ") ";
		}
		TelNo = TelNo + No;
		FriendlyNo = FriendlyNo + No;
	}
// make the call (<0 = error)
	if(lineMakeCall(Line, &Call, TelNo.c_str(), 0, &CallParams) < 0)
		return mError;
// wait for call to complete
	if(ProgressDlgOpen == true) return mSuccess;
	DialResult = mSuccess;
	while(1)
	{
		Application->ProcessMessages();
		if(FStatus == msConnected)
		{
			DialResult = mSuccess;
			break;
		}
		if(FStatus == msDisconnected)
		{
			DialResult = mError;
			break;
		}
		if(FStatus == msBusy)
		{
			DialResult = mBusy;
			break;
		}
		if(DialResult == mCancelled)
			break;
	}
	return DialResult;
}

/****************************************************************************

function hangup

Hangs up the current call

entry	void
exit	void

****************************************************************************/
void __fastcall TModem::Hangup()
{
	DialResult = mCancelled;
	if(DeviceCount == 0) return;
	if(Call == NULL) return;
	lineDrop(Call, NULL, 0);
	lineClose(Line);
	Shutdown();
}

/****************************************************************************

function listen

Enabled the TAPI device to listen for incoming calls

entry	DWORD number of rings after which to answer
exit	bool true or false

****************************************************************************/
bool __fastcall TModem::Listen(DWORD Rings/* = 3*/)
{
LINEEXTENSIONID extid;

	if(DeviceCount == 0) return false;
	if(Line) lineClose(Line);
// App and TAPI must negotiate to agree on a TAPI version (< 0 = error)
	if(lineNegotiateAPIVersion(LineApp, DeviceID, 0x00010000, 0x00090000,
								&tapiVersion, &extid) < 0)
		return false;
// open the line using a device that supports the parameters in the CallParams (<0 = error)
	if(lineOpen(LineApp, DeviceID, &Line, tapiVersion, 0, 0,
				LINECALLPRIVILEGE_OWNER,
				LINEMEDIAMODE_DATAMODEM | LINEMEDIAMODE_UNKNOWN,
				&CallParams) < 0)
		return false;
// tell TAPI we want RING messages
	if(lineSetStatusMessages(Line, LINEDEVSTATE_RINGING, 0) < 0)
		return false;
// setup ring count
	RingsToAnswer = Rings;
	Listening = true;
	return true;
}

/****************************************************************************

function Stoplisten

Stops listening for incoming calls

entry	DWORD number of rings after which to answer
exit	bool true or false

****************************************************************************/
void __fastcall TModem::StopListen()
{
	if(Listening == false) return;
	if(DeviceCount == 0) return;
	if(Call) lineDrop(Call, NULL, 0);
	lineClose(Line);
	Listening = false;
}

/****************************************************************************

function setdevice

Sets the device type used for making calls

entry	DWORD device type (index of device in list)
exit	void

****************************************************************************/
void __fastcall TModem::SetDevice(DWORD dev)
{
	if(dev >= DeviceCount) return;
	DeviceIndex = dev;
	DeviceID = (DWORD)ModemList->Objects[DeviceIndex];
	if(DialogOpen == true && OnDeviceChange != NULL)
		OnDeviceChange(this, DeviceIndex);
}

/****************************************************************************

function gethandle

returns the handle to the comms port. This handle is only valid if a call
is in progress.

entry
exit	HANDLE

****************************************************************************/
HANDLE __fastcall TModem::GetHandle(void)
{
	if(Connected == false) return INVALID_HANDLE_VALUE;
	else return hCommDevice;
}

/****************************************************************************

function freehandle

Closes the handle used by the comms port

entry	void
exit	void

****************************************************************************/
void __fastcall TModem::FreeHandle(void)
{
	Shutdown();
	if(hCommDevice != INVALID_HANDLE_VALUE) CloseHandle(hCommDevice);
	hCommDevice = INVALID_HANDLE_VALUE;
}

/****************************************************************************

function showconfiguredialog

Allows the user to configure the currently selected device

entry	void
exit	void

****************************************************************************/
bool __fastcall TModem::ShowConfigureDialog()
{
LINEEXTENSIONID extid;

	if(DeviceCount == 0) return false;
	if(Connected == true) return false;
// App and TAPI must negotiate to agree on a TAPI version (< 0 = error)
	if(lineNegotiateAPIVersion(LineApp, DeviceID, 0x00010000, 0x00090000,
								&tapiVersion, &extid) < 0)
		return false;
// open the line using a device that supports the parameters in the CallParams (<0 = error)
	else if(lineOpen(LineApp, DeviceID, &Line, tapiVersion, 0, 0,
						LINECALLPRIVILEGE_NONE, 0, &CallParams) < 0)
		return false;
// open dialog
	lineConfigDialog(DeviceID, Application->Handle, NULL);
// close line again
	lineClose(Line);
	return true;
}

/****************************************************************************

function showdialdialog

Allows the user to configure the dialling options and locations

entry	void
exit	void

****************************************************************************/
bool __fastcall TModem::ShowDialDialog()
{
LINEEXTENSIONID extid;

	if(DeviceCount == 0) return false;
	if(Connected == true) return false;
// App and TAPI must negotiate to agree on a TAPI version (< 0 = error)
	if(lineNegotiateAPIVersion(LineApp, DeviceID, 0x00010000, 0x00090000,
								&tapiVersion, &extid) < 0)
		return false;
// open dialog
	if(lineTranslateDialog(LineApp, DeviceID, tapiVersion, Application->Handle, NULL) < 0)
	{
		Application->MessageBox("Could not display Dialling Properties Dialog. Another application"
								" might already be displaying the dialog.", "Dialog Error",
								MB_ICONEXCLAMATION);
		return false;
	}
	return true;
}

/****************************************************************************

function userselectmodem

Opens a dialog in which the user may select the modem to use.

entry
exit

****************************************************************************/
void __fastcall TModem::UserSelectModem()
{
TModemSelectDlg *dlg;

	dlg = new TModemSelectDlg(this);
	dlg->DeviceIndex = DeviceIndex;
	DialogOpen = true;
	dlg->ShowModal();
	DialogOpen = false;
	delete dlg;
}

/****************************************************************************

function setcallproperties

Sets up the call properties before dialling or answering a call

entry
exit

****************************************************************************/
void __fastcall TModem::SetCallProperties()
{
LPDEVICECONFIG DeviceConfig;
DWORD Needed;

	DeviceConfig = new DEVICECONFIG;
	memset(DeviceConfig, 0, sizeof(DEVICECONFIG));
	DeviceConfig->vs.dwTotalSize = sizeof(DEVICECONFIG);
	lineGetDevConfig(DeviceID, (LPVARSTRING)DeviceConfig, "comm/datamodem");
	Needed = DeviceConfig->vs.dwNeededSize;
	delete DeviceConfig;
	if(Needed)
	{
		DeviceConfig = (LPDEVICECONFIG)new BYTE[Needed];
		memset(DeviceConfig, 0, Needed);
		DeviceConfig->vs.dwTotalSize = Needed;
		lineGetDevConfig(DeviceID, (LPVARSTRING)DeviceConfig, "comm/datamodem");
		if(FManualDial == moYes) DeviceConfig->fwOptions |= MANUAL_DIAL;
		if(FManualDial == moNo) DeviceConfig->fwOptions &= ~MANUAL_DIAL;
		if(FTerminalBefore == moYes) DeviceConfig->fwOptions |= TERMINAL_PRE;
		if(FTerminalBefore == moNo) DeviceConfig->fwOptions &= ~TERMINAL_PRE;
		if(FTerminalAfter == moYes) DeviceConfig->fwOptions |= TERMINAL_POST;
		if(FTerminalAfter == moNo) DeviceConfig->fwOptions &= ~TERMINAL_POST;
		if(FTrayMonitor == moYes) DeviceConfig->fwOptions |= LAUNCH_LIGHTS;
		if(FTrayMonitor == moNo) DeviceConfig->fwOptions &= ~LAUNCH_LIGHTS;
		lineSetDevConfig(DeviceID, (LPVOID)&DeviceConfig->dwSize, DeviceConfig->dwSize, "comm/datamodem");
		delete DeviceConfig;
	}
}

/****************************************************************************

function filldevicelist

Given a pointer to a TStrings object this function fills the list with device
names installed. This function also returns the index in the list of the
currently selected item, so a user can load his list with something like this:

	MyListBox->ItemIndex = Modem->FillDeviceList(MyListBox->Items);

This will also leave the listbox's index set to the current device.

entry	TStrings *
exit	void

****************************************************************************/
int __fastcall TModem::FillDeviceList(TStrings *s)
{
	s->Clear();
	s->Assign(ModemList);
	return (int)DeviceIndex;
}

/****************************************************************************

function fillcountrylist

Fills a list with the country names supported by TAPI

entry	TStrings * s
exit	index in list of current country, -1 if error

****************************************************************************/
int __fastcall TModem::FillCountryList(TStrings *s)
{
DWORD CountryID, oldid;
int index;
String Country;

	s->Clear();
	CountryID = 1;
// fill the list with the country names
	while(CountryID)
	{
		oldid = CountryID;
		Country = GetCountryName(CountryID);
		if(Country == "") break;
		index = s->Add(Country);
		s->Objects[index] = (TObject *)oldid;
	}
// return the index of the currently selected country
	return s->IndexOf(GetCountryName(FCountryID));
}

/****************************************************************************

function getcurrentcountryid

Sets up ID of the current country setup in the users dial location
properties.

entry	void
exit	void

****************************************************************************/
void __fastcall TModem::GetCurrentCountryID()
{
LPLINETRANSLATECAPS Caps;
LPLINELOCATIONENTRY Location;
DWORD Needed;

	FCountryID = 0;
	Caps = new LINETRANSLATECAPS;
	memset(Caps, 0, sizeof(LINETRANSLATECAPS));
	Caps->dwTotalSize = sizeof(LINETRANSLATECAPS);
	lineGetTranslateCaps(LineApp, tapiVersion, Caps);
	Needed = Caps->dwNeededSize;
	delete Caps;
	if(Needed)
	{
		Caps = (LPLINETRANSLATECAPS)new BYTE[Needed];
		Caps->dwTotalSize = Needed;
		lineGetTranslateCaps(LineApp, tapiVersion, Caps);
		Location = (LPLINELOCATIONENTRY)((BYTE *)Caps + Caps->dwLocationListOffset);
		FCountryID = Location->dwCountryID;
		delete Caps;
	}
}

/****************************************************************************

function filllocationlist

Fills a string list with all the locations enabled on the system

entry	TStrings *
exit	index of current location in list, or -1 if error

****************************************************************************/
int __fastcall TModem::FillLocationList(TStrings *s)
{
LPLINETRANSLATECAPS Caps;
LPLINELOCATIONENTRY Location;
DWORD Needed, x, LocationID;
long result;
int index;
char *b;

// get the location list from TAPI
	s->Clear();
	Needed = sizeof(LINETRANSLATECAPS);
	while(1)
	{
		Caps = (LPLINETRANSLATECAPS)new BYTE[Needed];
		if(!Caps) return -1;
		memset(Caps, 0, Needed);
		Caps->dwTotalSize = Needed;
		result = lineGetTranslateCaps(LineApp, tapiVersion, Caps);
		if(result == 0)
		{
			if(Caps->dwNeededSize > Caps->dwTotalSize)
				result = LINEERR_STRUCTURETOOSMALL;
		}
		if(result == 0) break;
		Needed = Caps->dwNeededSize;
		delete Caps;
		if(result != (long)LINEERR_STRUCTURETOOSMALL) return -1;
	}
// add the locations to the list
	Location = (LPLINELOCATIONENTRY)((BYTE *)Caps + Caps->dwLocationListOffset);
	for(x = 0; x < Caps->dwNumLocations; x++)
	{
		b = (char *)((BYTE *)Caps + Location->dwLocationNameOffset);
		index = s->Add(b);
		s->Objects[index] = (TObject *)Location->dwPermanentLocationID;
		Location++;
	}
	LocationID = Caps->dwCurrentLocationID;
	delete Caps;
// find the current location in the list
	for(index = 0; index < s->Count; index++)
	{
		if((DWORD)s->Objects[index] == LocationID)
			return index;
	}
	return -1;
}

/****************************************************************************

function getcurrentlocationid

Sets up ID and area code of the current dialling location

entry	void
exit	void

****************************************************************************/
void __fastcall TModem::GetCurrentLocationID()
{
LPLINETRANSLATECAPS Caps;
LPLINELOCATIONENTRY Location;
DWORD Needed;
long result;
DWORD x;

	FLocationID = 0;
	FAreaCode = "";
// get the location list from TAPI
	Needed = sizeof(LINETRANSLATECAPS);
	while(1)
	{
		Caps = (LPLINETRANSLATECAPS)new BYTE[Needed];
		if(!Caps) return;
		memset(Caps, 0, Needed);
		Caps->dwTotalSize = Needed;
		result = lineGetTranslateCaps(LineApp, tapiVersion, Caps);
		if(result == 0)
		{
			if(Caps->dwNeededSize > Caps->dwTotalSize)
				result = LINEERR_STRUCTURETOOSMALL;
		}
		if(result == 0) break;
		Needed = Caps->dwNeededSize;
		delete Caps;
		if(result != (long)LINEERR_STRUCTURETOOSMALL) return;
	}
	FLocationID = Caps->dwCurrentLocationID;
	Location = (LPLINELOCATIONENTRY)((BYTE *)Caps + Caps->dwLocationListOffset);
	for(x = 0; x < Caps->dwNumLocations; x++)
	{
		if(Location->dwPermanentLocationID == Caps->dwCurrentLocationID)
		{
			FAreaCode = (char *)((BYTE *)Caps + Location->dwCityCodeOffset);
			break;
		}
		Location++;
	}
	delete Caps;
}

/****************************************************************************

function shownumberdialog

displays the phone no dialog and allows the user to change it

entry	void
exit	bool true if OK, else false

****************************************************************************/
bool __fastcall TModem::ShowNumberDialog()
{
TDialOptionsDlg *dlg;
TModalResult r;

	dlg = new TDialOptionsDlg(this);
	if(!dlg) return false;
	r = dlg->ShowModal();
	delete dlg;
	return (r == mrOk);
}

/****************************************************************************

function getdevicename

returns the name of the selected modem device

entry	device ID
exit	String device name

****************************************************************************/
System::AnsiString __fastcall TModem::GetDeviceName(int device /*= -1*/)
{
String s;

	if(device < 0) device = DeviceIndex;
	if(device >= (int)DeviceCount) return System::AnsiString("Unknown device");
	return ModemList->Strings[device];
}

/****************************************************************************

function getcountryname

Given a country ID this functions returns the strng name of the country

entry	DWORD countryID
exit	String coutryname

****************************************************************************/
System::AnsiString __fastcall TModem::GetCountryName(DWORD &CountryID)
{
String Country;
LPLINECOUNTRYLIST List;
LPLINECOUNTRYENTRY Entry;
char *b;
DWORD Needed;
long result;

	Needed = sizeof(LINECOUNTRYLIST);
	Country = "";
// loop getting the info from TAPI until the structure we allocate is big enough
// to hold all the returned information
	while(1)
	{
		List = (LPLINECOUNTRYLIST)new BYTE[Needed];
		if(!List) return Country;
		memset(List, 0, Needed);
		List->dwTotalSize = Needed;
		if((result = lineGetCountry(CountryID, TAPI_CURRENT_VERSION, List)) == 0)
		{
			if(List->dwNeededSize > List->dwTotalSize)
				result = LINEERR_STRUCTURETOOSMALL;
		}
		if(result == 0) break;
		Needed = List->dwNeededSize;
		delete List;
		if(result != (long)LINEERR_STRUCTURETOOSMALL) return Country;
	}
// extract the country name from the structure
	Entry = (LPLINECOUNTRYENTRY)((BYTE *)List + List->dwCountryListOffset);
	b = (char *)((BYTE *)List + Entry->dwCountryNameOffset);
	Country = String(b) + String(" (") + IntToStr(Entry->dwCountryCode) +
				String(")");
	CountryID = Entry->dwNextCountryID;
	delete List;
	return Country;
}

/****************************************************************************

function getlocationname

Given the ID of a location this function returns the location name

entry 	DWORD locationID
exit	String location name

****************************************************************************/
System::AnsiString __fastcall TModem::GetLocationName(DWORD LocationID)
{
LPLINETRANSLATECAPS Caps;
LPLINELOCATIONENTRY Location;
DWORD Needed;
long result;
DWORD x;
String LocationName;

	LocationName = "";
// get the location list from TAPI
	Needed = sizeof(LINETRANSLATECAPS);
	while(1)
	{
		Caps = (LPLINETRANSLATECAPS)new BYTE[Needed];
		if(!Caps) return LocationName;
		memset(Caps, 0, Needed);
		Caps->dwTotalSize = Needed;
		result = lineGetTranslateCaps(LineApp, tapiVersion, Caps);
		if(result == 0)
		{
			if(Caps->dwNeededSize > Caps->dwTotalSize)
				result = LINEERR_STRUCTURETOOSMALL;
		}
		if(result == 0) break;
		Needed = Caps->dwNeededSize;
		delete Caps;
		if(result != (long)LINEERR_STRUCTURETOOSMALL) return LocationName;
	}
	Location = (LPLINELOCATIONENTRY)((BYTE *)Caps + Caps->dwLocationListOffset);
	for(x = 0; x < Caps->dwNumLocations; x++)
	{
		if(Location->dwPermanentLocationID == LocationID)
		{
			LocationName = (char *)((BYTE *)Caps + Location->dwLocationNameOffset);
			break;
		}
		Location++;
	}
	delete Caps;
	return LocationName;
}

/****************************************************************************

function getdevice

Returns the name of the currently selected device (modem)

entry
exit	String device name

****************************************************************************/
System::AnsiString __fastcall TModem::GetDevice()
{
	return GetDeviceName();
}

/****************************************************************************

function getcountry

Returns the name of the currently selected country. That is the country
to which the call will be made, not the country from which the call is made.

entry
exit	String countryname

****************************************************************************/
System::AnsiString __fastcall TModem::GetCountry()
{
	return GetCountryName(FCountryID);
}

/****************************************************************************

function getlocation

Returns the name of the current location as setup in the user's Control Panel

entry
exit	String location name

****************************************************************************/
System::AnsiString __fastcall TModem::GetLocation()
{
	return GetLocationName(FLocationID);
}

/****************************************************************************

function drawdeviceicon

given a pointer to an image object this function will draw an icon representing
the current device onto the canvas of the image. This function is defaulted
so calling it without a device ID will draw the icon for the currently selected
device.

entry	TCanvas *canvas, TColor background color [, int device]
exit	void

****************************************************************************/
void __fastcall TModem::DrawDeviceIcon(TImage *image, TColor bgcolor, int dev)
{
HICON hIcon;

	if(dev == -1) dev = Device;
	if(!lineGetIcon(dev, NULL, (LPHICON)&hIcon))
	{
		image->Width =
		image->Height = 32;
		image->Canvas->Brush->Color = bgcolor;
		DrawIconEx(	image->Canvas->Handle, 0, 0, hIcon, 0, 0, 0,
					image->Canvas->Brush->Handle, DI_DEFAULTSIZE | DI_NORMAL);
		image->Repaint();
	}
}

/****************************************************************************

function getcountrycode

Given a country ID this function returns the dialling code for the country

entry	DWORD country ID
exit	DWORD country code

****************************************************************************/
DWORD __fastcall TModem::GetCountryCode(DWORD CountryID)
{
LPLINECOUNTRYLIST List;
LPLINECOUNTRYENTRY Entry;
DWORD Needed, CountryCode;
long result;

	Needed = sizeof(LINECOUNTRYLIST);
// loop getting the info from TAPI until the structure we allocate is big enough
// to hold all the returned information
	while(1)
	{
		List = (LPLINECOUNTRYLIST)new BYTE[Needed];
		if(!List) return 0;
		memset(List, 0, Needed);
		List->dwTotalSize = Needed;
		if((result = lineGetCountry(CountryID, TAPI_CURRENT_VERSION, List)) == 0)
		{
			if(List->dwNeededSize > List->dwTotalSize)
				result = LINEERR_STRUCTURETOOSMALL;
		}
		if(result == 0) break;
		Needed = List->dwNeededSize;
		delete List;
		if(result != (long)LINEERR_STRUCTURETOOSMALL) return 0;
	}
// extract the country code from the structure
	Entry = (LPLINECOUNTRYENTRY)((BYTE *)List + List->dwCountryListOffset);
	CountryCode = Entry->dwCountryCode;
	delete List;
	return CountryCode;
}

/****************************************************************************

function getdiallablenumber

Converts a number, country code and area code into a diallable number by
running it through TAPI's translation API.

entry	DWORD CountryID, String AreaCode, String PhoneNo
exit	String diallable number

****************************************************************************/
System::AnsiString __fastcall TModem::GetDiallableNumber(DWORD CountryID,
														System::AnsiString AreaCode,
														System::AnsiString PhoneNo)
{
String Source, Dest;
DWORD CountryCode;
LPLINETRANSLATEOUTPUT DestNo;
DWORD Needed;
long result;

// get the country code for the country ID
	CountryCode = GetCountryCode(CountryID);
	Dest = "";
// build the source number
	Source = " ";
	if(AreaCode != "") Source = "(" + AreaCode + ") ";
	Source = "+" + IntToStr(CountryCode) + Source + PhoneNo;
// translate the destination number
	Needed = sizeof(LINETRANSLATEOUTPUT);
	while(1)
	{
		DestNo = (LPLINETRANSLATEOUTPUT) new BYTE[Needed];
		if(!DestNo) return Dest;
		memset(DestNo, 0, Needed);
		DestNo->dwTotalSize = Needed;
		result = lineTranslateAddress(LineApp, DeviceID, tapiVersion, Source.c_str(), 0, 0, DestNo);
		if(result == 0)
		{
			if(DestNo->dwNeededSize > DestNo->dwTotalSize)
				result = LINEERR_STRUCTURETOOSMALL;
		}
		if(result == 0) break;
		Needed = DestNo->dwNeededSize;
		delete DestNo;
		if(result != (long)LINEERR_STRUCTURETOOSMALL) return Dest;
	}
	Dest = (char *)DestNo + DestNo->dwDialableStringOffset;
	delete DestNo;
	return Dest;
}

