//---------------------------------------------------------------------------
//
// UART.CPP
//
// This is the implementation file for the component TUart.
//
// (c) D.C.Meintjies, 1997
//
// Use at your own risk, NO guarantees.
//
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "Uart.h"
//---------------------------------------------------------------------------
// This routine added by C++ Builder
//---------------------------------------------------------------------------
static inline TUart *ValidCtrCheck()
{
	return new TUart(NULL);
}
//---------------------------------------------------------------------------
// Constructor - Added by C++ Builder
//---------------------------------------------------------------------------
__fastcall TUart::TUart(TComponent* Owner)
	: TComponent(Owner)
{
// Set properties to default values...
	FOpen = false;
	FBaudRate = 9600;
	FPort = 1;
	FHandShake = hsNone;
	FParity = pNone;
	FStopBits = sbOne;
	FDataBits = 8;
	FDTR = false;
	FRTS = false;
	FAbort = false;
	FAutoOpen = false;
	hComm = INVALID_HANDLE_VALUE;
	FHandleErrors = true;
	FRxBuffSize = 1024;
	FTxBuffSize = 1024;
	LastCount = 0;
	HaltUpdate = false;
}
//---------------------------------------------------------------------------
//
// TUart destructor.
//
// Makes sure the port gets closed before destruction
//
//---------------------------------------------------------------------------
__fastcall TUart::~TUart()
{
	ClosePort();			/* close the port if it was open				*/
}
//---------------------------------------------------------------------------
// TUart registration - Added by C++ Builder
//---------------------------------------------------------------------------
namespace Uart
{
	void __fastcall Register()
	{
		TComponentClass classes[1] = {__classid(TUart)};
		RegisterComponents("MyStuff", classes, 0);
	}
}
//---------------------------------------------------------------------------
//
// function Loaded
//
// This function overrides the base class implementation. This function is
// called when a components properties have all been loaded from it's form file.
//
// Passed	void
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TUart::Loaded(void)
{
	TComponent::Loaded();		/* call base class implementation			*/
	if(FAutoOpen == true)
		Open = True;			/* open the port							*/
}
//---------------------------------------------------------------------------
//
// function ApplySettings
//
// This functions is called to apply the comms settings to the comms port.
//
// Passed	void
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TUart::ApplySettings()
{
	if(FOpen == false) return;			/* do nothing if port is closed		*/
	if(HaltUpdate == true) return;
	if(pPoll) pPoll->Suspend();					/* suspend port reading				*/
// fill in the new members of the structure...
	dcb.BaudRate = FBaudRate;
	dcb.fBinary = 1;
	if(FParity == pNone) dcb.fParity = 0;	/* disable parity if set to none	*/
	else dcb.fParity = 1;
	dcb.fOutxCtsFlow = 0;					/*									*/
	dcb.fOutxDsrFlow = 0;					/*									*/
	dcb.fOutX = 0;							/*  default to handshaking disabled	*/
	dcb.fInX = 0;							/*									*/
	if(FDTR)
		dcb.fDtrControl = DTR_CONTROL_ENABLE;/*									*/
	else
		dcb.fDtrControl = DTR_CONTROL_DISABLE;/*								*/
	if(FRTS)
		dcb.fRtsControl = RTS_CONTROL_ENABLE;/*									*/
	else
		dcb.fRtsControl = RTS_CONTROL_DISABLE;/*								*/
	switch(FHandShake)
	{
		case hsXonXoff:
			dcb.fOutX = 1;
			dcb.fInX = 1;
			break;
		case hsCtsRts:
			dcb.fOutxCtsFlow = 1;
			dcb.fRtsControl = RTS_CONTROL_HANDSHAKE;
			break;
		case hsDtrDsr:
			dcb.fOutxDsrFlow = 1;
			dcb.fDtrControl = DTR_CONTROL_HANDSHAKE;
			break;
	}
	dcb.fDsrSensitivity = 0;		/* ignore DSR during receive				*/
	dcb.fTXContinueOnXoff = true;	/* Continue TX if our RX buffer is full		*/
	dcb.fErrorChar = 0;				/* no Error  Character used					*/
	dcb.fNull = 0;					/* Don't ignore NULL bytes					*/
	dcb.fAbortOnError = 0;			/* don't abort I/O on error					*/
	dcb.ByteSize = (BYTE)FDataBits;
	dcb.Parity = FParity;
	dcb.StopBits = FStopBits;
// save new DCB settings
	if(!SetCommState(hComm, (LPDCB)&dcb))
	{
		UartError(UE_SETCOMMSTATE,"Error setting comms state");
		if(pPoll) pPoll->Resume();
		return;
	}
// resume the polling thread...
	if(pPoll) pPoll->Resume();
}
//---------------------------------------------------------------------------
//
// function ClearErrors
//
// Clears errors that may exist for the com port
//
// Passed	void
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TUart::ClearErrors()
{
	DWORD Errors;
	if(FOpen == false) return;
	ClearCommError(hComm, (LPDWORD)&Errors, NULL);
}
//---------------------------------------------------------------------------
//
// function SetBaudRate
//
// Sets the Baudrate property
//
// Passed	DWORD baudrate
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TUart::SetBaudRate(DWORD baud)
{
	if(baud != FBaudRate)
	{
		FBaudRate = baud;
		ApplySettings();
	}
}
//---------------------------------------------------------------------------
//
// function SetHandShake
//
// Sets up the FHandShake property
//
// Passed	THandShake hs
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TUart::SetHandShake(THandShake hs)
{
	if(hs != FHandShake)
	{
		FHandShake = hs;
		ApplySettings();
	}
}
//---------------------------------------------------------------------------
//
// function SetStopBits
//
// Sets up the FStopBits property
//
// Passed	TStopBits sb
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TUart::SetStopBits(TStopBits sb)
{
	if(sb != FStopBits)
	{
		FStopBits = sb;
		ApplySettings();
	}
}
//---------------------------------------------------------------------------
//
// function SetSerialPort
//
// Sets the FPort property. If this is called while the port is open the port
// is closed and re-opened with the new value. If the new value fails to open,
// the old port is not re-opened.
//
// Passed	int port
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TUart::SetSerialPort(int port)
{
bool wasopen;

	if(port != FPort)
	{
		wasopen = FOpen;
		ClosePort();
		FPort = port;
		Open = wasopen;
	}
}
//---------------------------------------------------------------------------
//
// function SetDataBits
//
// Sets the FDataBits property. Values below 6 are ignored.
//
// Passed	int db
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TUart::SetDataBits(int db)
{
	if(db != FDataBits && db >= 4)
	{
		FDataBits = db;
		ApplySettings();
	}
}
//---------------------------------------------------------------------------
//
// function SetParity
//
// Thes up the FParity property
//
// Passed	TParity par
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TUart::SetParity(TParity par)
{
	if(par != FParity)
	{
		FParity = par;
		ApplySettings();
	}
}
//---------------------------------------------------------------------------
//
// function SetDTR
//
// Sets the FDTR property and also the line to the indecated state
//
// Passed	bool setdtr
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TUart::SetDTR(bool setdtr)
{
bool result;

	FDTR = setdtr;
	if(FOpen == true && FHandShake != hsDtrDsr)
	{
		if(FDTR == true) result = EscapeCommFunction(hComm, SETDTR);
		else result = EscapeCommFunction(hComm, CLRDTR);
		if(result == false)
			UartError(UE_SETDTR,"Error setting DTR line.");
	}
}
//---------------------------------------------------------------------------
//
// function SetRTS
//
// Sets the FRTS and the RTS line to the indicated state
//
// Passed	bool setrts
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TUart::SetRTS(bool setrts)
{
bool result;

	FRTS = setrts;
	if(FOpen == true && FHandShake != hsCtsRts)
	{
		if(FRTS == true) result = EscapeCommFunction(hComm, SETRTS);
		else result = EscapeCommFunction(hComm, CLRRTS);
		if(result == false)
			UartError(UE_SETRTS,"Error setting RTS line.");
	}
}
//---------------------------------------------------------------------------
//
// function Open
//
// Opens the port and applies all the port settings
//
// Passed	void
// Returns	bool TRUE if okay, else FALSE
//
//---------------------------------------------------------------------------
void __fastcall TUart::SetOpen(bool o)
{
	if(o == FOpen) return;		/* already opened / closed				*/
	if(o) OpenPort();
	else ClosePort();
	ApplySettings();
// set buffer sizes, flush the buffers and clear current errors...
	SetBufferSizes();
	ClearErrors();
	Flush();
}
//---------------------------------------------------------------------------
//
// function ClosePort
//
// closes the communications port
//
// Passed	void
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall::TUart::ClosePort()
{
	if(FOpen == false) return;
	if(pPoll) pPoll->Terminate();
	CloseHandle(hComm);
	FOpen = false;
}
//---------------------------------------------------------------------------
//
// function Openport
//
// Opens the vomms port for read/write NON-OVERLAPPED I/O
//
// Passed	void
// Returns	false if open fails
//
//---------------------------------------------------------------------------
bool __fastcall TUart::OpenPort()
{
char str[100];

	if(ComponentState.Contains(csDesigning)) return false;/* use only at run-time*/
	ClosePort();										/* in case it's open	*/
	sprintf(str,"COM%d", FPort);						/* build port's filename*/
	hComm = CreateFile( str,							/* name of port			*/
						GENERIC_READ | GENERIC_WRITE,	/* read/write access	*/
						NULL,							/* no sharing			*/
						NULL,							/* no security for Win95*/
						OPEN_EXISTING,					/* port must exist		*/
						FILE_ATTRIBUTE_NORMAL,			/* normal i/o on port	*/
						NULL);							/* no templates in Win95*/
	if(hComm == INVALID_HANDLE_VALUE)
	{
		sprintf(str,"COM%d failed to open.\nThe port may be in use by another application,\nor it does not exist.", FPort);
		UartError(UE_OPEN, str);
		FOpen = false;
		return false;
	}
	FOpen = true;
	if(!GetCommState(hComm, (LPDCB)&dcb))
	{									/* get comms DCB settings			*/
		UartError(UE_GETCOMMSTATE,"Error getting Comms State");
		return false;
	}
	if(OnReceive || OnLineChange)
	{
		pPoll = new TPollThread(this, hComm, true);		/* create the reading thread	*/
		pPoll->Resume();
	}
	else pPoll == NULL;
	return true;
}
//---------------------------------------------------------------------------
//
// function UartError
//
// This is the global error routines for any Uart-related errors. If the user has
// supplied an event handler then his handler is called and passed the error code
// and a string describing the error.
//
// Passed	int errorcode, char * error message
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TUart::UartError(int ErrCode, char *errmsg)
{
	if(ComponentState.Contains(csDesigning)) return; /* ignore in design mode */
	if(OnError) OnError(this, ErrCode, (LPCSTR)errmsg); /* call user's event	*/
	if(FHandleErrors != true) return;			/* error handling disabled, exit */
	MessageBeep(MB_ICONEXCLAMATION);			/* draw user's attention		*/
	Application->MessageBox(errmsg, "Communications Error", MB_ICONEXCLAMATION | MB_OK);
}
//---------------------------------------------------------------------------
//
// function SetTxBuff
//
// Sets the size of the port's transmit buffer
//
// Passed	DWORD new size
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TUart::SetTxBuff(DWORD size)
{
	if(FTxBuffSize != size)
	{
		FTxBuffSize = size;
		SetBufferSizes();
	}
}
//---------------------------------------------------------------------------
//
// function SetRxBuff
//
// Sets the size of the port's Receive buffer
//
// Passed	DWORD size
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TUart::SetRxBuff(DWORD size)
{
	if(FRxBuffSize != size)
	{
		FRxBuffSize = size;
		SetBufferSizes();
	}
}
//---------------------------------------------------------------------------
//
// function
//
// Passed
// Returns
//
//---------------------------------------------------------------------------
void __fastcall TUart::SetRxNotifyLevel(DWORD size)
{
	FRxNotifyLevel = size;
}
//---------------------------------------------------------------------------
//
// function SendStr
//
// Sends a null-terminated string out the port
//
// Passed	LPCSTR string to send
// Returns	true if okay, else false
//
//---------------------------------------------------------------------------
bool __fastcall TUart::SendStr(char *str, DWORD maxtime)
{
	return SendBuff((void *)str, (DWORD)strlen(str), maxtime);
}
//---------------------------------------------------------------------------
//
// function SendByte
//
// Sends 1 byte to the serial port
//
// Passed	BYTE to send
// Returns	bool TRUE if okay, else false
//
//---------------------------------------------------------------------------
bool __fastcall TUart::SendByte(BYTE c, DWORD maxtime)
{
COMSTAT cs;
DWORD Errors, written;

	if(FOpen == false) return false;
	FAbort = false;
	do
	{
		ClearCommError(hComm, (LPDWORD)&Errors, (LPCOMSTAT)&cs);
	} while(FTxBuffSize - cs.cbOutQue <= 0);
	if(WriteFile(hComm, (LPCVOID)&c, 1, (LPDWORD)&written, NULL) == true)
	{
		do
		{
			ClearCommError(hComm, (LPDWORD)&Errors, (LPCOMSTAT)&cs);
		} while(cs.cbOutQue > 0);
		return true;
	}
	return false;
}
//---------------------------------------------------------------------------
//
// function SendString
//
// Sends an String out the serial port
//
// Passed	String s
// Returns	bool TRUE if okay, else false
//
//---------------------------------------------------------------------------
bool __fastcall TUart::SendString(String s, DWORD maxtime)
{
	return SendBuff((void *)s.c_str(), s.Length(), maxtime);
}
//---------------------------------------------------------------------------
//
// function SendBuff
//
// Sends a buffer of bytes out the serial port
//
// Passed	void
// Returns	void
//
//---------------------------------------------------------------------------
bool __fastcall TUart::SendBuff(void *p, DWORD len, DWORD maxtime)
{
DWORD written, TimeTick, Errors, Space, Remaining;
COMSTAT cs;
BYTE *pByte;

	if(FOpen == false) return false;
	FAbort = false;
	TimeTick = GetTickCount();
	Remaining = len;
	pByte = (BYTE *)p;
	while(1)
	{
		ClearCommError(hComm, (LPDWORD)&Errors, (LPCOMSTAT)&cs);
		Space = FTxBuffSize - cs.cbOutQue;	/* get buffer space				*/
		if(Space > Remaining) Space = Remaining;
		Remaining -= Space;
		if(WriteFile(hComm, (LPCVOID)pByte, Space, (LPDWORD)&written, NULL) == false)
			return false;
		if(!Remaining) break;				/* all done						*/
		pByte += Space;
		Application->ProcessMessages();		/* keep App alive				*/
		if((GetTickCount() - TimeTick) > maxtime) return false; /* timeout	*/
		if(FAbort == true) return false;
	}
	return true;
}
//---------------------------------------------------------------------------
//
// function flush
//
// Empties the receive and transmit queues of the uart
//
// Passed	void
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TUart::Flush()
{
	if(FOpen == false) return;
	PurgeComm(hComm, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);
}
//---------------------------------------------------------------------------
//
// function setbuffersizes
//
// Sets the sizes of the Transmit and Receive buffers
//
// Passed	void
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TUart::SetBufferSizes()
{
	if(FOpen == false) return;	/* ignore if port not open				*/
	SetupComm(hComm, FRxBuffSize, FTxBuffSize);
}
//---------------------------------------------------------------------------
//
// function GetCTS
//
// Returns the state of the CTS (Clear To Send) line. This is invoked by
// accessing the CTS property
//
// Passed	void
// Returns	bool true or false
//
//---------------------------------------------------------------------------
bool __fastcall TUart::GetCTS()
{
DWORD mstat;

	if(FOpen == false) return false;
	GetCommModemStatus(hComm, (LPDWORD)&mstat);
	return ((mstat & MS_CTS_ON) != 0);
}
//---------------------------------------------------------------------------
//
// function GetDSR
//
// Returns the state of the DSR (Data Set Ready) line. This is invoked by
// accessing the DSR property
//
// Passed	void
// Returns	bool true or false
//
//---------------------------------------------------------------------------
bool __fastcall TUart::GetDSR()
{
DWORD mstat;

	if(FOpen == false) return false;
	GetCommModemStatus(hComm, (LPDWORD)&mstat);
	return ((mstat & MS_DSR_ON) != 0);
}
//---------------------------------------------------------------------------
//
// function GetCD
//
// Returns the state of the CD (Carrier Detect) line This is invoked by
// accessing the CD property
//
// Passed	void
// Returns	bool true or false
//
//---------------------------------------------------------------------------
bool __fastcall TUart::GetCD()
{
DWORD mstat;

	if(FOpen == false) return false;
	GetCommModemStatus(hComm, (LPDWORD)&mstat);
	return ((mstat & MS_RLSD_ON) != 0);
}
//---------------------------------------------------------------------------
//
// function GetRING
//
// Returns the state of the RI (Ring Indicator) line. This is invoked by
// accessing the RING property
//
// Passed	void
// Returns	bool true or false
//
//---------------------------------------------------------------------------
bool __fastcall TUart::GetRING()
{
DWORD mstat;

	if(FOpen == false) return false;
	GetCommModemStatus(hComm, (LPDWORD)&mstat);
	return ((mstat & MS_RING_ON) != 0);
}
//---------------------------------------------------------------------------
//
// function GetModemLines
//
// Returns the state of the Modem status lines. This is invoked by
// accessing the ModemLines property
//
// Passed	void
// Returns	DWORD containing the modem status lines
//
//---------------------------------------------------------------------------
DWORD __fastcall TUart::GetModemLines()
{
DWORD mstat;

	if(FOpen == false) return false;
	GetCommModemStatus(hComm, (LPDWORD)&mstat);
	return mstat;
}
//---------------------------------------------------------------------------
//
// function RxReady
//
// Called by the polling thread to indicate that characters have arrived on
// the serial port
//
// Passed	DWORD number of bytes in receive que
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TUart::RxReady(DWORD count)
{
	if(count >= FRxNotifyLevel)
	{
		if(OnReceive) OnReceive(this, count);
	}
}
//---------------------------------------------------------------------------
//
// function LineChange
//
// Called by the polling thread to indicate that one of the modem input lines
// has changed state
//
// Passed	void
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TUart::LineChange()
{
	if(OnLineChange)
		OnLineChange(this, GetModemLines());
}
//---------------------------------------------------------------------------
//
// function RxCount
//
// Returns the number of bytes available in the Receive queue
//
// Passed	void
// Returns	DWORD count
//
//---------------------------------------------------------------------------
DWORD __fastcall TUart::GetRxCount()
{
COMSTAT cs;
DWORD Errors;

	if(FOpen == false) return 0;
	ClearCommError(hComm, (LPDWORD)&Errors, (LPCOMSTAT)&cs);
	return cs.cbInQue;
}
//---------------------------------------------------------------------------
//
// function TxCount
//
// Returns the number of bytes still waiting in the transmit queue
//
// Passed	void
// Returns	DWORD count
//
//---------------------------------------------------------------------------
DWORD __fastcall TUart::GetTxCount()
{
COMSTAT cs;
DWORD Errors;

	if(FOpen == false) return 0;
	ClearCommError(hComm, (LPDWORD)&Errors, (LPCOMSTAT)&cs);
	return cs.cbOutQue;
}
//---------------------------------------------------------------------------
//
// function WaitFor
//
// Waits for a specific character to arrive on the serial port. All characters
// arriving before the requested one are discarded
//
// Passed	BYTE to wait for, DWORD maxtime (in milliseconds)
// Returns	bool TRUE or FALSE
//
//---------------------------------------------------------------------------
bool __fastcall TUart::WaitFor(BYTE c, DWORD maxtime)
{
DWORD TimeTick, BytesRead;
BYTE b;
bool retval;

	if(FOpen == false) return false;
	TimeTick = GetTickCount();
	if(pPoll) pPoll->Suspend();			/* disable thread while we read			*/
	FAbort = false;
	retval = false;
	while(1)
	{
		while(RxCount)
		{
			ReadFile(hComm, (LPVOID)&b, 1, (LPDWORD)&BytesRead, NULL);
			if(b == c)
			{
				retval = true;
				break;
			}
			TimeTick = GetTickCount();	/* reload timeout					*/
		}
		if(retval == true) break;
		if((GetTickCount() - TimeTick) > maxtime) break;	/* timeout		*/
		if(FAbort == true) break;
		Application->ProcessMessages();
	}
	if(pPoll) pPoll->Resume();
	return retval;
}
//---------------------------------------------------------------------------
//
// function ReceiveStr
//
// Receives bytes from the serial port. This functions is the same as
// ReceiveBuff, except it appends a NULL character to the end of the string.
//
// Passed	char * to store the bytes at
//			DWORD number of bytes to read
//			DWORD max time to wait in milliseconds
// Returns	DWORD number of bytes read
//
//---------------------------------------------------------------------------
DWORD __fastcall TUart::ReceiveStr(char *str, DWORD len, DWORD maxtime)
{
DWORD count, TimeTick, BytesToRead, Remaining, BytesRead;
char *b;

	if(FOpen == false) return 0;
	TimeTick = GetTickCount();
	count = 0;
	if(pPoll) pPoll->Suspend();			/* disable thread while we read			*/
	Remaining = len;
	b = str;
	FAbort = false;
	while(Remaining)
	{
		if(RxCount)
		{
			BytesToRead = RxCount;
			if(BytesToRead > Remaining) BytesToRead = Remaining;
			ReadFile(hComm, (LPVOID)b, BytesToRead, (LPDWORD)&BytesRead, NULL);
			Remaining -= BytesRead;
			count += BytesRead;
			b += BytesRead;
			if(!Remaining) break;
			TimeTick = GetTickCount();	/* reload timeout					*/
		}
		if((GetTickCount() - TimeTick) > maxtime) break;	/* timeout		*/
		if(FAbort == true) break;
		Application->ProcessMessages();
	}
	*b = NULL;
	if(pPoll) pPoll->Resume();
	return count;
}
//---------------------------------------------------------------------------
//
// function ReceiveBuff
//
// Receives bytes from the serial port
//
// Passed	void * to store the bytes at
//			DWORD number of bytes to read
//			DWORD max time to wait in milliseconds
// Returns	DWORD number of bytes read
//
//---------------------------------------------------------------------------
DWORD __fastcall TUart::ReceiveBuff(void *buff, DWORD len, DWORD maxtime)
{
DWORD count, TimeTick, BytesToRead, Remaining, BytesRead;
BYTE *b;

	if(FOpen == false) return 0;
	TimeTick = GetTickCount();
	count = 0;
	if(pPoll) pPoll->Suspend();			/* disable thread while we read			*/
	Remaining = len;
	b = (BYTE *)buff;
	FAbort = false;
	while(Remaining)
	{
		if(RxCount)
		{
			BytesToRead = RxCount;
			if(BytesToRead > Remaining) BytesToRead = Remaining;
			ReadFile(hComm, (LPVOID)b, BytesToRead, (LPDWORD)&BytesRead, NULL);
			Remaining -= BytesRead;
			count += BytesRead;
			b += BytesRead;
			if(!Remaining) break;
			TimeTick = GetTickCount();	/* reload timeout					*/
		}
		if((GetTickCount() - TimeTick) > maxtime) break;	/* timeout		*/
		if(FAbort == true) break;
		Application->ProcessMessages();
	}
	if(pPoll) pPoll->Resume();
	return count;
}
//---------------------------------------------------------------------------
//
// function ReceiveString
//
// Receives a String object from the serial port
//
// Passed	DWORD length, DWORD maxtime
// Returns	String, or NULL if error
//
//---------------------------------------------------------------------------
String __fastcall TUart::ReceiveString(DWORD len, DWORD maxtime)
{
String s;
DWORD TimeTick, BytesToRead, Remaining, BytesRead;
BYTE b[201];

	if(FOpen == false) return NULL;
	TimeTick = GetTickCount();
	if(pPoll) pPoll->Suspend();			/* disable thread while we read			*/
	Remaining = len;
	FAbort = false;
	while(Remaining)
	{
		if(RxCount)
		{
			BytesToRead = RxCount;
			if(BytesToRead > 200) BytesToRead = 200;
			if(BytesToRead > Remaining) BytesToRead = Remaining;
			ReadFile(hComm, (LPVOID)b, BytesToRead, (LPDWORD)&BytesRead, NULL);
			b[BytesRead] = NULL;
			s = s + (char *)b;
			Remaining -= BytesRead;
			if(!Remaining) break;
			TimeTick = GetTickCount();	/* reload timeout					*/
		}
		if((GetTickCount() - TimeTick) > maxtime) break;	/* timeout		*/
		if(FAbort == true) break;
		Application->ProcessMessages();
	}
	if(pPoll) pPoll->Resume();
	return s;
}
//---------------------------------------------------------------------------
//
// function ReceiveByte
//
// Receives a character from the serial port
//
// Passed	BYTE &c, DWORD maxtime
// Returns	true or false
//
//---------------------------------------------------------------------------
bool __fastcall TUart::ReceiveByte(BYTE &c, DWORD maxtime)
{
DWORD TimeTick, BytesRead, LastTick;
bool retval;

	if(FOpen == false) return false;
	if(RxCount)
	{
		ReadFile(hComm, (LPVOID)&c, 1, (LPDWORD)&BytesRead, NULL);
		return true;
	}

	TimeTick = GetTickCount();
	LastTick = TimeTick;
	if(pPoll) pPoll->Suspend();			/* disable thread while we read			*/
	retval = false;
	while(1)
	{
		if(RxCount)
		{
			ReadFile(hComm, (LPVOID)&c, 1, (LPDWORD)&BytesRead, NULL);
			retval = true;
			break;
		}
		if(FAbort == true) break;
		if((GetTickCount() - TimeTick) > maxtime) break;	/* timeout		*/
		if((GetTickCount() - LastTick) > 5)
		{
			Application->ProcessMessages();
			LastTick = GetTickCount();
		}
	}
	if(pPoll) pPoll->Resume();
	return retval;
}
//---------------------------------------------------------------------------
//
// function Abort
//
// Aborts any pending Transmit / Receive operation
//
// Passed	void
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TUart::SetAbort(bool a)
{
	if(FOpen == false) return;
	FAbort = a;
}
//---------------------------------------------------------------------------
//
// function BeginUpdate
//
// This functions sets a flag for the component not to update
// the UART itself everytime a property is changed. When EndUpdate is called the
// changes made take effect       
//
// Passed	void
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TUart::BeginUpdate()
{
	HaltUpdate = true;
}
//---------------------------------------------------------------------------
//
// function
//
// Passed
// Returns
//
//---------------------------------------------------------------------------
void __fastcall TUart::EndUpdate()
{
	HaltUpdate = false;
	ApplySettings();
}
//---------------------------------------------------------------------------
//
// function
//
// Passed
// Returns
//
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//
// function
//
// Passed
// Returns
//
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//
// function
//
// Passed
// Returns
//
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//
// function
//
// Passed
// Returns
//
//---------------------------------------------------------------------------
/****************************************************************************
TUartThread
****************************************************************************/
//---------------------------------------------------------------------------
//
// function TPollThread
//
// Constructor for a thread that continually polls the serial port for
// characters arriving and modem line status change.
//
// Passed
// Returns
//
//---------------------------------------------------------------------------
__fastcall TPollThread::TPollThread(TUart *uart, HANDLE ComFile, bool CreateSuspended)
	: TThread(CreateSuspended)
{
	FreeOnTerminate = true;			/* delete ourselves when we terminate	*/
	hComm = ComFile;				/* save handle to open COM file			*/
	pUart = uart;					/* save ptr to main component			*/
	LastLevel = 0;					/* no character seen yet				*/
	GetCommModemStatus(hComm, (LPDWORD)&LineStatus); /* save initial status	*/
}
//---------------------------------------------------------------------------
//
// function Execute
//
// Main loop of the TPollThread thread
//
// Passed	void
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TPollThread::Execute()
{
COMSTAT cs;
DWORD Errors, tmp;

	while(!Terminated)
	{
		ClearCommError(hComm, (LPDWORD)&Errors, (LPCOMSTAT)&cs);
		tmp = LastLevel;
		LastLevel = cs.cbInQue;
		if(/*LastLevel != tmp &&*/ LastLevel != 0)
			Synchronize(SendEvent);
		GetCommModemStatus(hComm, (LPDWORD)&tmp);
		if(tmp != LineStatus)
		{
			LineStatus = tmp;
			Synchronize(SendLineStatus);
		}
	}
}
//---------------------------------------------------------------------------
//
// function SendEvent
//
// This function is used by the Synchronize methiod to tell the UART component
// that a character has arrived on the serial port.
//
// Passed	void
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TPollThread::SendEvent()
{
COMSTAT cs;
DWORD Errors;

	ClearCommError(hComm, (LPDWORD)&Errors, (LPCOMSTAT)&cs);
	LastLevel = cs.cbInQue;
	pUart->RxReady(LastLevel);
}
//---------------------------------------------------------------------------
//
// function SendLineStatus
//
// This function is used by the Synchronize method to tell the UART component
// that the modem lines have changed state
//
// Passed	void
// Returns	void
//
//---------------------------------------------------------------------------
void __fastcall TPollThread::SendLineStatus()
{
	pUart->LineChange();
}

